import type { Querystring, RequestArgs } from "../types/request";
import type { GeneralSchema, SuccessResponse } from "../types/schema";
export declare type HttpMethod = "GET" | "POST" | "DELETE" | "PATCH" | "PUT";
export declare function changeBearerToken(newToken: string): void;
export declare function fullFetch(path: string, query?: Querystring, init?: RequestInit): Promise<Response>;
export declare type FullFetch = typeof fullFetch;
export interface FetchInfo {
    path: string;
    method?: HttpMethod;
    query?: Querystring;
    body?: unknown;
    headers?: Record<string, string>;
}
export declare type JsonFetchResult<TResp> = TResp;
declare type RejectHandler<TRej> = (reason: any) => TRej | PromiseLike<TRej>;
export declare class JsonFetchResultPromiseLike<T extends GeneralSchema> implements PromiseLike<SuccessResponse<T>> {
    private promise;
    private httpErrorHandler;
    constructor(promise: Promise<Response>);
    then<TSuc = SuccessResponse<T>, TRej = never>(onfulfilled?: ((value: SuccessResponse<T>) => TSuc | PromiseLike<TSuc>) | null, onrejected?: ((reason: any) => TRej | PromiseLike<TRej>) | null): Promise<TSuc | TRej>;
    /**
     * Attach an error handler specific to a status code.
     *
     * The handler will be called when this code is received.
     * The return value of the handler will be called with onrejected.
     *
     * @param code http status code
     * @param handler handler for this type of code
     * @returns A Promise
     */
    httpError<Code extends number>(code: Code, handler: (err: T["responses"][Code]) => any): JsonFetchResultPromiseLike<T>;
    catch<TRej = never>(onrejected?: RejectHandler<TRej> | null): Promise<SuccessResponse<T> | TRej>;
    finally(onfinally?: () => any): Promise<unknown>;
}
/**
 * Fetch and returns as json.
 * @param info the fetch info
 * @throws {Response} If the request is successful but response is not json,
 * the response will be thrown
 * @throws {JsonFetchError} If the statusCode is not [200, 300), a error will be thrown
 */
export declare function jsonFetch<T extends GeneralSchema>(info: FetchInfo, signal?: AbortSignal): JsonFetchResultPromiseLike<T>;
export declare type JsonFetch = typeof jsonFetch;
export declare function fromApi<TSchema extends GeneralSchema>(method: HttpMethod, url: string): (args: RequestArgs<TSchema>, signal?: AbortSignal) => JsonFetchResultPromiseLike<TSchema>;
export {};
